import logging

class Log():

    __instance = None
    __logger = None

    @staticmethod
    def getInstance():
        if Log.__instance == None:
            return Log()
            
        return Log.__instance

    def __init__(self):

        logging.basicConfig(format='%(asctime)s - %(name)s - %(process)s - %(levelname)s - %(message)s', level=logging.NOTSET)

        if Log.__instance != None:
            raise Exception("Can not instantiate more than once!")

        # Create a custom logger
        logger = logging.getLogger('symbl')

        # Create handlers
        c_handler = logging.StreamHandler()
        c_handler.setLevel(logging.NOTSET)

        self.__logger = logger
    
    def set_level(self, level):
        self.__logger.setLevel(level)

    def info(self, message: str, data=None):
        if data != None:
            self.__logger.info("{}, {}".format(message, data))
        else:
            self.__logger.info(message)

    def debug(self, data, *args):
        if args != None and len(args) > 0:
            self.__logger.debug(data, args[0])
        else:
            self.__logger.debug(data)

    def warning(self, data, *args):
        if args != None and len(args) > 0:
            self.__logger.warning(data, args[0])
        else:
            self.__logger.warning(data)
    
    def error(self, data, *args):
        if args != None and len(args) > 0:
            self.__logger.error(data, args[0])
        else:
            self.__logger.error(data)



from symbl.Conversations import Conversation
from symbl import AuthenticationToken
from symbl_rest import AsyncApi as async_api_rest
from symbl.utils.Logger import Log

def initialize_api_client(function):
    def wrapper(*args, **kw):
        credentials = None
        self = args[0]
        
        if 'credentials' in kw:
            credentials = kw['credentials']

        api_client = AuthenticationToken.get_api_client(credentials)
        self.async_api_rest = async_api_rest(api_client)

        return function(*args, **kw)
    
    return wrapper

def correct_boolean_values(dictionary: dict):
    for key in dictionary:
        if dictionary[key] == True:
            dictionary[key] = "true"
        elif dictionary[key] == False:
            dictionary[key] = "false"
    return dictionary

class Video():

    def __init__(self, api_client=None):
        '''
            It will initialize the Analysis class
            with the object of Initialize Class
        '''
        self.__async_api_rest = async_api_rest(api_client)

    @initialize_api_client
    def process_file(self, file_path:str, credentials=None, content_type:str='video/mp4', wait:bool=True, parameters={}):
        '''
            video files to be analyzed
            returns Conversation object
        '''
        if file_path == None:
            raise ValueError("Please enter a valid file_path")

        file = open(file_path, 'rb')
        video_file = file.read()
        response = self.__async_api_rest.add_video(body=video_file, content_type=content_type, **correct_boolean_values(parameters))
        Log.getInstance().info("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)


    @initialize_api_client
    def process_url(self, url:str, credentials=None, wait:bool=True, parameters={}):
        '''
            url of audio file to be analyzed
            returns Conversation object
        '''

        if url == None:
            raise ValueError("Please enter a valid url.")

        response = self.__async_api_rest.add_video_url(body={ 'url': url }, **correct_boolean_values(parameters))
        Log.getInstance().info("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)


    @initialize_api_client
    def append_file(self, file_path:str, conversation_id:str, credentials=None, content_type:str='video/mp4', wait:bool=True, parameters={}):
        '''
            video files to be appended
            returns Conversation object
        '''
        if file_path == None:
            raise ValueError("Please enter a valid file_path")

        if conversation_id == None or len(conversation_id) == 0:
            raise ValueError("Please enter a valid conversation_id")

        file = open(file_path, 'rb')
        video_file = file.read()
        response = self.__async_api_rest.append_video(body=video_file, content_type=content_type, conversation_id=conversation_id, **correct_boolean_values(parameters))
        Log.getInstance().info("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)


    @initialize_api_client
    def append_url(self, url : str, conversation_id:str, credentials=None, wait:bool=True, parameters={}):
        '''
            url of video file to be appended
            returns Conversation object
        '''
        if url == None:
            raise ValueError("Please enter a valid url")

        if conversation_id == None or len(conversation_id) == 0:
            raise ValueError("Please enter a valid conversation_id")

        response = self.__async_api_rest.append_video_url(body={ 'url': url }, conversation_id=conversation_id, **correct_boolean_values(parameters))
        Log.getInstance().info("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)


from symbl.utils.Logger import Log
from symbl.Conversations import Conversation
from symbl import AuthenticationToken
from symbl_rest import AsyncApi as async_api_rest

def initialize_api_client(function):
    def wrapper(*args, **kw):
        credentials = None
        self = args[0]
        
        if 'credentials' in kw:
            credentials = kw['credentials']

        api_client = AuthenticationToken.get_api_client(credentials)
        self.__async_api_rest = async_api_rest(api_client)

        return function(*args, **kw)
    
    return wrapper

def correct_boolean_values(dictionary: dict):
    for key in dictionary:
        if dictionary[key] == True:
            dictionary[key] = "true"
        elif dictionary[key] == False:
            dictionary[key] = "false"
    return dictionary
    
class Text():

    def __init__(self, api_client=None):
        '''
            It will initialize the Analysis class
            with the object of Initialize Class
        '''
        self.__async_api_rest = async_api_rest(api_client)

    @initialize_api_client
    def process(self, payload : dict, credentials=None, wait: bool = True, parameters={}):
        '''
            Text payload to be analyzed
            returns Conversation object
        '''
        
        if payload == None:
            raise ValueError("Please enter a valid payload.")

        response = self.__async_api_rest.add_text(body=payload, **correct_boolean_values(parameters))
        Log.getInstance().info("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)

    @initialize_api_client
    def append(self, payload:dict, conversation_id:str, credentials=None, wait: bool = True, parameters={}):
        '''
            Text payload to be appended
            returns Conversation object
        '''
        if payload == None:
            raise ValueError("Text Payload can not be None")

        if conversation_id == None or len(conversation_id) == 0:
            raise ValueError("Please enter a valid conversationId")

        response = self.__async_api_rest.append_text(conversation_id, body=payload, **correct_boolean_values(parameters))
        Log.getInstance().info("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)


from symbl import AuthenticationToken
from symbl_rest import ConversationsApi as conversations_api_rest

def initialize_api_client(function):
    def wrapper(*args, **kw):
        credentials = None
        self = args[0]
        
        if 'credentials' in kw:
            credentials = kw['credentials']

        api_client = AuthenticationToken.get_api_client(credentials)
        self.async_api_rest = conversations_api_rest(api_client)

        return function(*args, **kw)
    
    return wrapper

def correct_boolean_values(dictionary: dict):
    for key in dictionary:
        if dictionary[key] == True:
            dictionary[key] = "true"
        elif dictionary[key] == False:
            dictionary[key] = "false"
    return dictionary

class ConversationsApi():

    def __init__(self, api_client=None):
        '''
            It will initialize the ConversationsApi class
        '''
        
        self.api_client = api_client
        self.conversations_api_rest = conversations_api_rest(api_client)

    @initialize_api_client
    def get_action_items(self, conversation_id, credentials=None):
        return self.conversations_api_rest.get_action_items_by_conversation_id(conversation_id)

    @initialize_api_client
    def get_follow_ups(self, conversation_id, credentials=None ):  
        return self.conversations_api_rest.get_follow_ups_by_conversation_id(conversation_id)

    @initialize_api_client
    def get_insights(self, conversation_id, credentials=None):  
        return self.conversations_api_rest.get_insights_by_conversation_id(conversation_id)
  
    @initialize_api_client      
    def get_members(self, conversation_id, credentials=None):  
        return self.conversations_api_rest.get_members_by_conversation_id(conversation_id)
  
    @initialize_api_client      
    def get_messages(self, conversation_id, credentials=None, parameters={}):  
        return self.conversations_api_rest.get_messages_by_conversation_id(conversation_id, **correct_boolean_values(parameters))
  
    @initialize_api_client      
    def get_questions(self, conversation_id, credentials=None):  
        return self.conversations_api_rest.get_questions_by_conversation_id(conversation_id)
  
    @initialize_api_client      
    def get_topics(self, conversation_id, credentials=None, parameters={}):
        return self.conversations_api_rest.get_topics_by_conversation_id(conversation_id, **correct_boolean_values(parameters))



from symbl.utils.Logger import Log
from symbl.Conversations import Conversation
from symbl.utils.Decorators import wrap_keyboard_interrupt
from symbl.utils.Threads import Thread
from symbl.configs.configs import SYMBL_WEBSOCKET_BASE_PATH, X_API_KEY_HEADER
from symbl.AuthenticationToken import get_api_header
import json
import websocket

class Connection():

    def __init__(self, conversationId: str, connectionId: str, resultWebSocketUrl: str, eventUrl: str, credentials=None):
        
        self.conversation = Conversation(conversationId)
        self.credentials = credentials
        self.connectionId = connectionId
        self.resultWebSocketUrl = resultWebSocketUrl
        self.eventUrl = eventUrl
        access_token = get_api_header(self.credentials).get(X_API_KEY_HEADER)
        self.header = "{}:{}".format(X_API_KEY_HEADER, access_token)
        self.connection = None

    def __on_message_handler(self, data, event_callbacks):
        try:
            if data != None and type(data) == str:  
                json_data = json.loads(data)
                if 'type' in json_data and json_data['type'] in event_callbacks:
                    event_callbacks[json_data['type']](data) 
        except Exception as error:
            Log.getInstance().error(error)

    def subscribe(self, event_callbacks: dict):
        try:
            if event_callbacks != None or event_callbacks != {}:
                Log.getInstance().info("Eshtablishing connection")
                self.connection = websocket.WebSocketApp(url=SYMBL_WEBSOCKET_BASE_PATH + self.connectionId, header=[self.header])
                Log.getInstance().info("Connection Eshtablished")

                self.connection.on_message = lambda this, data : self.__on_message_handler(data=data, event_callbacks=event_callbacks)
                self.connection.on_error = lambda self, error : Log.getInstance().error(error)

                self.connection.run_forever()
            else:
                Log.getInstance().error("Can not subscribe to empty events")
        except (KeyboardInterrupt, SystemExit):
            self.stop()
            Log.getInstance().error("Exiting")

    def stop(self):
        self.connection.send(data=str({
            'type': 'stop_request'
        }))
        self.connection.close()


from enum import Enum

class JobStatus(Enum):
    SCHEDULED = "scheduled"
    IN_PROGRESS = "in_progress"
    FAILED = "failed"
    COMPLETED = "completed"

    def __eq__(self, other): 
        if not isinstance(other, JobStatus):
            # don't attempt to compare against unrelated types
            raise ValueError("Can not compare JobStatus type object with {} type object".format(type(other)))

        return self.value == other.value


from symbl.Conversations import Conversation

import unittest
from unittest.mock import Mock, patch

from types import SimpleNamespace


class ConversationsTest(unittest.TestCase):

    def test_get_messages_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"messages":[{"id":"6125275260649472","text":"Hey, is this a question?","from":{},"startTime":"2021-05-27T13:19:25.270Z","endTime":"2021-05-27T13:19:26.770Z","conversationId":"6549352073920512","phrases":[]}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_messages", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.messages())

    def test_get_action_items_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"actionItems":[{"id":"4755377456414720","text":"Steve needs to complete the analysis by next Monday.","type":"action_item","score":0.9765610554943914,"messageIds":["5590459222065152"],"entities":[{"type":"daterange","text":"by next monday","offset":39,"end":"2021-06-07"},{"type":"person","text":"Steve","offset":0,"value":{"assignee":True,"id":"98d11b11-4a0a-4bdd-b2e1-d50238f605d6","name":"Steve","userId":"Steve@example.com"}}],"phrases":[],"from":{"id":"98d11b11-4a0a-4bdd-b2e1-d50238f605d6","name":"Steve","userId":"Steve@example.com"},"definitive":True,"assignee":{"id":"98d11b11-4a0a-4bdd-b2e1-d50238f605d6","name":"Steve","email":"Steve@example.com"}}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_action_items", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.action_items())

    def test_get_follow_ups_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"followUps":[{"id":"4529863081852928","text":"Steve needs to follow up about this next Monday.","type":"follow_up","score":1,"messageIds":["5399026486738944"],"entities":[{"type":"date","text":"next monday","offset":38,"value":"2021-06-07"},{"type":"person","text":"Steve","offset":0,"value":{"assignee":True,"id":"e2219a6c-ec5e-4412-94d7-46f565ba5eb7","name":"Steve","userId":"Steve@example.com"}}],"phrases":[],"from":{"id":"e2219a6c-ec5e-4412-94d7-46f565ba5eb7","name":"Steve","userId":"Steve@example.com"},"definitive":True,"assignee":{"id":"e2219a6c-ec5e-4412-94d7-46f565ba5eb7","name":"Steve","email":"Steve@example.com"}}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_follow_ups", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.follow_ups())

    def test_get_insights_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"insights":[{"id":"4529863081852928","text":"Steve needs to follow up about this next Monday.","type":"follow_up","score":1,"messageIds":["5399026486738944"],"entities":[{"type":"date","text":"next monday","offset":38,"value":"2021-06-07"},{"type":"person","text":"Steve","offset":0,"value":{"assignee":True,"id":"e2219a6c-ec5e-4412-94d7-46f565ba5eb7","name":"Steve","userId":"Steve@example.com"}}],"phrases":[],"from":{"id":"e2219a6c-ec5e-4412-94d7-46f565ba5eb7","name":"Steve","userId":"Steve@example.com"},"definitive":True,"assignee":{"id":"e2219a6c-ec5e-4412-94d7-46f565ba5eb7","name":"Steve","email":"Steve@example.com"}}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_insights", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.insights())

    def test_get_members_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"members":[{"id":"25f497bd-2800-4e2b-bc71-e143a8f71d6b","name":"Steve","email":"steve@abccorp.com"}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_members", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.members())

    def test_get_questions_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"questions":[{"id":"4784246146203648","text":"Will you be reviewint the analysis by next Monday?","type":"question","score":0.9833490590134427,"messageIds":["4580678118146048"],"from":{"id":"ca0fcd23-dc63-4d42-894a-43ca84650f6f","name":"Steve","userId":"Steve@example.com"}}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_questions", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.questions())

    def test_get_topics_should_succeed_given_valid_conversation_id(self):
        demo_response = SimpleNamespace(**{"topics":[{"id":"5506508885327872","text":"video","type":"topic","score":0.064,"messageIds":["5823121492803584","4673825691140096"],"parentRefs":[]}]})
        conversations_object = Conversation("6549352073920512")

        with patch("symbl.AuthenticationToken.get_api_client", Mock(return_value="None")), patch("symbl.ConversationsApi.get_topics", Mock(return_value=demo_response)):
            self.assertEqual(demo_response, conversations_object.topics())

if __name__ == '__main__':
    unittest.main()



from symbl.Connection import Connection
from symbl.telephony_api.TelephonyValidators import validateActions, validateEndpoint
from symbl_rest import ConnectionToEndpointApi as telephony_api_rest
from symbl import AuthenticationToken


def initialize_api_client(function):
    def wrapper(*args, **kw):
        credentials = None
        self = args[0]
        
        if 'credentials' in kw:
            credentials = kw['credentials']

        api_client = AuthenticationToken.get_api_client(credentials)
        self.api_client = api_client
        self.async_api_rest = telephony_api_rest(api_client)

        return function(*args, **kw)
    
    return wrapper
class TelephonyApi():
    def __init__(self, api_client=None):
        '''
            It will initialize the Telephony class
        '''
        
        self.api_client = api_client
        self.telephony_api_rest = telephony_api_rest(api_client)

    def validateAndConnectToEndpoint(self, body, credentials=None):
        if body == None:
            raise ValueError('endpoint configuration is required.')

        if "endpoint" not in body :
            raise ValueError('Please enter the endpoint you want to connect.')

        validateEndpoint(body["endpoint"])
        
        validateActions(body["actions"])

        data = self.telephony_api_rest.connect_to_endpoint(body)
        connectionObject = Connection(connectionId=data.connection_id, conversationId=data.conversation_id, resultWebSocketUrl=data.result_web_socket_url, eventUrl=data.event_url, credentials=credentials)
        
        return connectionObject

    @initialize_api_client
    def start_pstn(self, phone_number, dtmf=None, credentials=None, actions={}, data={}):
        body = dict()
        body = {
            "operation": "start", 
            "endpoint": {
                "type": "pstn",
                "phoneNumber": phone_number,
                "dtmf": dtmf
            }, 
            "actions": actions,
            "data": data,
            "pushSpeakerEvents": True
        }

        return self.validateAndConnectToEndpoint(body, credentials)

    @initialize_api_client
    def start_sip(self, uri, audio_config={}, credentials=None, actions={}, data={}):
        body = dict()

        if audio_config == {}:

            audio_config = { 
                "sampleRate": 16000,
                "sampleSize": "16"
            }

        body = {
            "operation": "start", 
            "endpoint": {
                "type": "sip",
                "uri": uri,
                "audioConfig": audio_config
            }, 
            "actions": actions,
            "data": data,
            # "pushSpeakerEvents": True
        }

        return self.validateAndConnectToEndpoint(body)

    @initialize_api_client
    def stop(self, connection_id):
        body = dict()
        body["operation"] = "stop"
        body["connectionId"] = connection_id


        if connection_id == None:
            raise ValueError('ConnectionId is invalid, Please enter a valid connectionId to stop')

        return self.telephony_api_rest.connect_to_endpoint(body)



# coding: utf-8

"""
    symbl.ai APIs

    <h1>Symbl's APIs for external consumers.</h1> <h2>Language Insights API</h2> Symbl's Language Insights API provides an interface for applications to perform the analysis on the raw text and get insights from it. The API automatically detects sentence boundaries and punctuates the sentences, and also returns the updated messages in the response. Conversations are the most unstructured piece of information that we represent information in, and which most of the leads to lot of loss of information by not being able to capture them correctly.<br/> Language Insights API focuses on understanding such texts and generate the useful and important information from them. <br/> Currently the API supports detection of the Action Items in any type of unstructured text. In future the same API will also have support to detect \"Information\" and \"Event\", where Information is any informational piece and Event is a reference to something that has happened in the past.<br/> <h2>Telephony Integration</h2> Symbl can currently integrate with two types of telephony endpoints: 1. SIP trunks<br/> 2. PSTN endpoints<br/> Results are sent via HTTP WebHooks as and when they are available.<br/> <h2>Flow</h2> 1. External Application invokes REST API to join a meeting/session, with the mode (SIP/PSTN) and joining details<br/> 2. Symbl joins the meeting via SIP or PSTN integration<br/> 3. Symbl continuously processes the audio stream received<br/> 4. Symbl calls WebHook whenever transcription results are available<br/>  # noqa: E501

    Contact: info@symbl.ai
"""

from setuptools import setup, find_packages  # noqa: H301

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools


setup(
    name="symbl",
    version="1.0.7",
    description="symbl.ai SDK",
    author_email="info@symbl.ai",
    url="",
    keywords=["Symbl.ai SDK"],
    install_requires=["symbl_rest >= 1.0.0", "websocket-client >= 0.59.0", "sounddevice >= 0.4.1", "numpy"],
    packages=find_packages(),
    include_package_data=True,
    long_description="""\
    &lt;h1&gt;Symbl&#x27;s APIs for external consumers.&lt;/h1&gt; &lt;h2&gt;Language Insights API&lt;/h2&gt; Symbl&#x27;s Language Insights API provides an interface for applications to perform the analysis on the raw text and get insights from it. The API automatically detects sentence boundaries and punctuates the sentences, and also returns the updated messages in the response. Conversations are the most unstructured piece of information that we represent information in, and which most of the leads to lot of loss of information by not being able to capture them correctly.&lt;br/&gt; Language Insights API focuses on understanding such texts and generate the useful and important information from them. &lt;br/&gt; Currently the API supports detection of the Action Items in any type of unstructured text. In future the same API will also have support to detect \&quot;Information\&quot; and \&quot;Event\&quot;, where Information is any informational piece and Event is a reference to something that has happened in the past.&lt;br/&gt; &lt;h2&gt;Telephony Integration&lt;/h2&gt; Symbl can currently integrate with two types of telephony endpoints: 1. SIP trunks&lt;br/&gt; 2. PSTN endpoints&lt;br/&gt; Results are sent via HTTP WebHooks as and when they are available.&lt;br/&gt; &lt;h2&gt;Flow&lt;/h2&gt; 1. External Application invokes REST API to join a meeting/session, with the mode (SIP/PSTN) and joining details&lt;br/&gt; 2. Symbl joins the meeting via SIP or PSTN integration&lt;br/&gt; 3. Symbl continuously processes the audio stream received&lt;br/&gt; 4. Symbl calls WebHook whenever transcription results are available&lt;br/&gt;  # noqa: E501
    """
)



from __future__ import absolute_import
from symbl.utils import Log
from symbl_rest import  JobsApi, ApiClient

from symbl.async_api.Audio import Audio
from symbl.async_api.Video import Video
from symbl.async_api.Text import Text

from symbl.Connection import Connection
from symbl.streaming_api.StreamingConnection import StreamingConnection

from symbl.conversations_api.ConversationsApi import ConversationsApi
from symbl.telephony_api.TelephonyApi import TelephonyApi
from symbl.streaming_api.StreamingApi import StreamingApi
from symbl.jobs_api.Job import Job
from symbl.jobs_api.JobStatus import JobStatus

Audio = Audio()
Video = Video()
Text = Text()
Conversations = ConversationsApi()
Telephony = TelephonyApi()
Streaming = StreamingApi()
